<?php namespace Premmerce\Optimizer\Cache;

class FileSystemCache extends Cache
{
    private $path;

    private $loaded = array();

    public function __construct($dir)
    {
        $this->path = rtrim($dir) . '/';
        if (! is_dir($this->path)) {
            mkdir($this->path, 0777, true);
        }
    }

    /**
     * Fetches a value from the cache.
     *
     * @param string $key The unique key of this item in the cache.
     * @param mixed $default Default value to return if the key does not exist.
     *
     * @return mixed The value of the item from the cache, or $default in case of cache miss.
     *
     * @throws \Psr\SimpleCache\InvalidArgumentException
     *   MUST be thrown if the $key string is not a legal value.
     */
    public function get($key, $default = null)
    {
        if ($this->loadItem($key)) {
            return $this->loaded[$key];
        }

        return $default;
    }

    /**
     * Persists data in the cache, uniquely referenced by a key with an optional expiration TTL time.
     *
     * @param string $key The key of the item to store.
     * @param mixed $value The value of the item to store, must be serializable.
     * @param null|int|\DateInterval $ttl Optional. The TTL value of this item. If no value is sent and
     *                                      the driver supports TTL then the library may set a default value
     *                                      for it or let the driver take care of that.
     *
     * @return bool True on success and false on failure.
     *
     * @throws \Psr\SimpleCache\InvalidArgumentException
     *   MUST be thrown if the $key string is not a legal value.
     */
    public function set($key, $value, $ttl = null)
    {
        $this->validateKey($key);

        $expiration = $ttl ? time() + $ttl : null;

        $result = false !== file_put_contents($this->path . $key, serialize(array($expiration, $value)));

        if ($result) {
            $this->loaded[$key] = $value;
        }

        return $result;
    }

    /**
     * Delete an item from the cache by its unique key.
     *
     * @param string $key The unique cache key of the item to delete.
     *
     * @return bool True if the item was successfully removed. False if there was an error.
     *
     * @throws \Psr\SimpleCache\InvalidArgumentException
     *   MUST be thrown if the $key string is not a legal value.
     */
    public function delete($key)
    {
        $this->validateKey($key);

        unset($this->loaded[$key]);

        return unlink($this->path . $key);
    }

    /**
     * Wipes clean the entire cache's keys.
     *
     * @return bool True on success and false on failure.
     */
    public function clear()
    {
        $files  = glob($this->path . '*');
        $result = true;
        foreach ($files as $file) {
            $result = $result && unlink($file);
        }

        $this->loaded = array();

        return $result;
    }

    /**
     * Determines whether an item is present in the cache.
     *
     * NOTE: It is recommended that has() is only to be used for cache warming type purposes
     * and not to be used within your live applications operations for get/set, as this method
     * is subject to a race condition where your has() will return true and immediately after,
     * another script can remove it making the state of your app out of date.
     *
     * @param string $key The cache item key.
     *
     * @return bool
     *
     * @throws \Psr\SimpleCache\InvalidArgumentException
     *   MUST be thrown if the $key string is not a legal value.
     */
    public function has($key)
    {
        return $this->loadItem($key);
    }

    /**
     * @param $key
     *
     * @return bool
     * @throws InvalidArgumentException
     */
    protected function loadItem($key)
    {
        $this->validateKey($key);

        if (isset($this->loaded[$key])) {
            return true;
        }

        if (! file_exists($this->path . $key)) {
            return false;
        }

        $data = @unserialize(file_get_contents($this->path . $key));

        if (! is_array($data)) {
            return false;
        }

        if (is_int($data[0]) && $data[0] <= time()) {
            return false;
        }

        $this->loaded[$key] = $data[1];

        return true;
    }
}
