<?php namespace Premmerce\Optimizer\Frontend;

use Premmerce\Optimizer\Admin\Settings;
use Premmerce\Optimizer\OptimizerPlugin;
use Premmerce\Optimizer\WooCommerce\ProductFactory;
use Premmerce\Optimizer\WooCommerce\Proxy\ProductVariableLite;
use Premmerce\SDK\V2\FileManager\FileManager;
use Psr\SimpleCache\CacheInterface;

/**
 * Class Frontend
 *
 * @package Premmerce\Optimizer\Frontend
 */
class Frontend
{
    /**
     * @var CacheInterface
     */
    private $cache;

    /**
     * @var FileManager
     */
    private $fileManager;

    /**
     * @var string
     */
    private $activeItemClass = 'current-menu-item';

    /**
     * Frontend constructor.
     *
     * @param FileManager $fileManager
     * @param CacheInterface $cache
     */
    public function __construct(FileManager $fileManager, CacheInterface $cache)
    {
        $this->cache       = $cache;
        $this->fileManager = $fileManager;

        $option = get_option(Settings::OPTION);

        add_filter('premmerce_optimizer_cache', function () {
            return $this->cache;
        });

        if (! empty($option['optimize_woocommerce'])) {
            add_action('woocommerce_init', array($this, 'replaceWooProductFactory'));
            add_filter('pre_transient_wc_attribute_taxonomies', array($this, 'cacheAttributes'));
        }

        if (! empty($option['cache_menus'])) {
            add_filter('wp_nav_menu', array($this, 'cacheMenu'), 10, 2);
            add_filter('pre_wp_nav_menu', array($this, 'getMenuFromCache'), 10, 2);
            add_filter('wp_nav_menu_objects', array($this, 'removeActiveItems'), 10);
            add_action('wp_enqueue_scripts', array($this, 'registerAssets'));
        }

        if (! empty($option['optimize_variable_products'])) {
            add_filter('woocommerce_product_class', function ($class) {
                switch ($class) {
                    case \WC_Product_Variable::class:
                        return ProductVariableLite::class;
                        break;
                }

                return $class;
            });
        }
    }

    /**
     * @param array $items
     *
     * @return mixed
     */
    public function removeActiveItems($items)
    {
        foreach ($items as $item) {
            $key = array_search('current-menu-item', $item->classes);
            if (false !== $key) {
                $item->current = false;
                unset($item->classes[$key]);
            }
        };

        return $items;
    }

    /**
     * @param mixed $menu
     * @param \stdClass $args
     *
     * @return mixed
     * @throws \Psr\SimpleCache\InvalidArgumentException
     */
    public function getMenuFromCache($menu, $args)
    {
        $key = $this->getMenuCacheKey($args);

        if ($this->cache->has($key)) {
            $menu = $this->cache->get($key);
        }

        return $menu;
    }

    /**
     * @param string $nav_menu
     * @param \stdClass $args
     *
     * @return mixed
     * @throws \Psr\SimpleCache\InvalidArgumentException
     */
    public function cacheMenu($nav_menu, $args)
    {
        $key = $this->getMenuCacheKey($args);

        $this->cache->set($key, $nav_menu);

        return $nav_menu;
    }

    /**
     * Replace woo product factory
     */
    public function replaceWooProductFactory()
    {
        WC()->product_factory = new ProductFactory();
    }

    /**
     * Cache attributes from transients
     * @return bool|mixed
     */
    public function cacheAttributes()
    {
        if (isset($GLOBALS['getting_premmerce_wc_attribute_taxonomies'])) {
            return false;
        }

        if ($tax = wp_cache_get('premmerce_wc_attribute_taxonomies')) {
            return $tax;
        }

        $GLOBALS['getting_premmerce_wc_attribute_taxonomies'] = true;

        $tax = get_transient('wc_attribute_taxonomies');

        unset($GLOBALS['getting_premmerce_wc_attribute_taxonomies']);

        wp_cache_set('premmerce_wc_attribute_taxonomies', $tax);

        return $tax;
    }

    /**
     * Register assets
     */
    public function registerAssets()
    {
        $js = $this->fileManager->locateAsset('frontend/js/premmerce-optimizer.js');

        wp_enqueue_script('premmerce-optimizer', $js, array('jquery'), OptimizerPlugin::VERSION);
        wp_localize_script('premmerce-optimizer', 'premmerceOptimizer', array(
            'activeMenuItemClass' => $this->activeItemClass,
        ));
    }

    protected function getMenuCacheKey($args)
    {
        $key = 'menu_' . $args->theme_location . (isset($args->menu->term_id) ? $args->menu->term_id : '') . $this->getLanguageCode();

        return $key;
    }

    /**
     * @return string|null
     */
    protected function getLanguageCode()
    {
        if (defined('ICL_LANGUAGE_CODE')) {
            return ICL_LANGUAGE_CODE;
        }

        return null;
    }
}
